#include <Drivers.h>
#include <KernelExport.h>
#include <OS.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <unistd.h>

/*

	/dev/config/dprintf driver for BeOS
	(c) 2002, mmu_man, revol@free.fr
	(de)activates kernel syslog via dprintf
	usage:
		make driverinstall
		rescan dprintf
		cat /dev/config/dprintf
		echo "1" >/dev/config/dprintf # to enable dprintf
		echo "0" >/dev/config/dprintf # to disable dprintf
	licenced under MIT licence
	NOT SMP-safe, NOT threadsafe (well, not sure)
*/

/*
	NOTES:
	I saw such a driver somewhere on the web, but can't get my hands on it, 
	so here is my version.
	
	The code that checks if it is enabled (in read()), enables it temporarily.
*/


const char * device_names[]={"config/dprintf_enabled", NULL};
extern device_hooks dprintf_enabled_hooks;

int32 api_version = B_CUR_DRIVER_API_VERSION;

typedef struct {
	int dummy;
} cookie_t;



status_t init_hardware(void) {
dprintf("dprintf:init_hardware()\n");
	return B_OK;
}

status_t init_driver(void) {
dprintf("dprintf:init_driver()\n");
	return B_OK;
}

void uninit_driver(void) {
dprintf("dprintf:uninit_driver()\n");
}

const char **publish_devices() {
	return device_names;
}

device_hooks *find_device(const char *name) {
	(void)name;
	return &dprintf_enabled_hooks;
}

status_t dprintf_enabled_open(const char *name, uint32 flags, cookie_t **cookie) {
	(void)name; (void)flags;
	*cookie = (void*)malloc(sizeof(cookie_t));
	if (*cookie == NULL) {
		dprintf("dprintf_open : error allocating cookie\n");
		goto err0;
	}
	memset(*cookie, 0, sizeof(cookie_t));
	return B_OK;
	
err1:
	dprintf("dprintf_open : cleanup : will free cookie\n");
	free(*cookie);
	*cookie = NULL;
err0:
	return B_ERROR;
}

status_t dprintf_enabled_close(void *cookie) {
	(void)cookie;
	return B_OK;
}

status_t dprintf_enabled_free(cookie_t *cookie) {
	free(cookie);
	return B_OK;
}

status_t dprintf_enabled_ioctl(cookie_t *cookie, uint32 op, void *data, size_t len) {
	(void)cookie; (void)op; (void)data; (void)len;
	return B_ERROR;
}

status_t dprintf_enabled_read(cookie_t *cookie, off_t position, void *data, size_t *numbytes) {
	char buff[3] = "0\n";
	bool was_enabled;
	int numread = 0;

	(void)position;
	if (position > 1)
		goto ERROR_EOF;
	if (*numbytes < 1)
		goto ERROR_EOF; /* wow, funky ! */
	/* better enable it than loose any kernel message if we disable it :) */
	was_enabled = set_dprintf_enabled(true);
	set_dprintf_enabled(was_enabled);
	buff[0] = was_enabled?'1':'0';
	while (position < 2 && position < *numbytes) {
		*(char *)data++ = buff[position++];
		numread++;
	}
	*numbytes = numread;
	return B_OK;

ERROR_EOF:
	*numbytes = 0;
	return B_OK;
}

status_t dprintf_enabled_write(void *cookie, off_t position, const void *data, size_t *numbytes) {
	bool ena;
	(void)cookie; (void)position; (void)data;
//dprintf("dprintf_enabled_write(, %ld, , %ld)\n", (long)position, (long)*numbytes);
	if (position > 0)
		return B_OK;
	if (*numbytes < 1)
		goto ERROR_EOF;
	if (*(char *)data == '1' || *(char *)data == 'y')
		ena = true;
	else if (*(char *)data == '0' || *(char *)data == 'n')
		ena = false;
	else
		return EINVAL;
	set_dprintf_enabled(ena);
	return B_OK;

ERROR_EOF:
	*numbytes = 0;
	return B_ERROR;
}

device_hooks dprintf_enabled_hooks={
	(device_open_hook)dprintf_enabled_open,
	dprintf_enabled_close,
	(device_free_hook)dprintf_enabled_free,
	(device_control_hook)dprintf_enabled_ioctl,
	(device_read_hook)dprintf_enabled_read,
	dprintf_enabled_write,
	NULL,
	NULL,
	NULL,
	NULL
};
