#include <Drivers.h>
#include <KernelExport.h>
#include <OS.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <unistd.h>

/*

	/dev/config/sysinfo driver for BeOS
	(c) 2002, mmu_man, revol@free.fr
	some kernel infos...
	usage:
		make driverinstall
		rescan sysinfo
		cat /dev/misc/sysinfo
	licenced under MIT licence
	NOT SMP-safe, NOT threadsafe (well, not sure)
*/

/*
*/

extern uint32 max_swap_size;
extern uint32 swap_size;
extern uint32 used_swap_size;

const char * device_names[]={"misc/sysinfo", NULL};
extern device_hooks sysinfo_hooks;

int32 api_version = B_CUR_DRIVER_API_VERSION;

#define THE_BUFF_SIZE 256

typedef struct {
	char buff[THE_BUFF_SIZE];
	int len;
} cookie_t;



status_t init_hardware(void) {
dprintf("sysinfo:init_hardware()\n");
	return B_OK;
}

status_t init_driver(void) {
dprintf("sysinfo:init_driver()\n");
	return B_OK;
}

void uninit_driver(void) {
dprintf("sysinfo:uninit_driver()\n");
}

const char **publish_devices() {
	return device_names;
}

device_hooks *find_device(const char *name) {
	(void)name;
	return &sysinfo_hooks;
}

status_t sysinfo_open(const char *name, uint32 flags, cookie_t **cookie) {
	(void)name; (void)flags;
	*cookie = (void*)malloc(sizeof(cookie_t));
	if (*cookie == NULL) {
		dprintf("sysinfo_open : error allocating cookie\n");
		goto err0;
	}
	memset(*cookie, 0, sizeof(cookie_t));
	sprintf((*cookie)->buff, "max_swap_size\t%ld\nswap_size\t%ld\nused_swap_size\t%ld\n", max_swap_size, swap_size, used_swap_size);
	(*cookie)->len = strlen((*cookie)->buff);
	return B_OK;
	
err1:
	dprintf("sysinfo_open : cleanup : will free cookie\n");
	free(*cookie);
	*cookie = NULL;
err0:
	return B_ERROR;
}

status_t sysinfo_close(void *cookie) {
	(void)cookie;
	return B_OK;
}

status_t sysinfo_free(cookie_t *cookie) {
	free(cookie);
	return B_OK;
}

status_t sysinfo_ioctl(cookie_t *cookie, uint32 op, void *data, size_t len) {
	(void)cookie; (void)op; (void)data; (void)len;
	return B_ERROR;
}

status_t sysinfo_read(cookie_t *cookie, off_t position, void *data, size_t *numbytes) {
	int numread = 0;

	if (position > cookie->len)
		goto ERROR_EOF;
	while (position < cookie->len && position < *numbytes) {
		*(char *)data++ = cookie->buff[position++];
		numread++;
	}
	*numbytes = numread;
	return B_OK;

ERROR_EOF:
	*numbytes = 0;
	return B_OK;
}

status_t sysinfo_write(void *cookie, off_t position, const void *data, size_t *numbytes) {
	bool ena;
	(void)cookie; (void)position; (void)data;
//dprintf("sysinfo_write(, %ld, , %ld)\n", (long)position, (long)*numbytes);
ERROR_EOF:
	*numbytes = 0;
	return B_ERROR;
}

device_hooks sysinfo_hooks={
	(device_open_hook)sysinfo_open,
	sysinfo_close,
	(device_free_hook)sysinfo_free,
	(device_control_hook)sysinfo_ioctl,
	(device_read_hook)sysinfo_read,
	sysinfo_write,
	NULL,
	NULL,
	NULL,
	NULL
};
